with Ada.Text_IO;
procedure Main is

   type Digit_T is new Integer range 0 .. 9;

   function Convert (S : String) return Digit_T;
   function Convert (I : Digit_T) return String;

   function Convert (S : String) return Digit_T is
   begin
      for D in Digit_T loop
         if Convert (D) = S then
            return D;
         end if;
      end loop;

      raise Constraint_Error with "not a Digit_T: """ & S & """";
   end Convert;

   function Convert (I : Digit_T) return String is
     (case I is
      when 0 => "zero",
      when 1 => "one",
      when 2 => "two",
      when 3 => "three",
      when 4 => "four",
      when 5 => "five",
      when 6 => "six",
      when 7 => "seven",
      when 8 => "eight",
      when 9 => "nine"
      );

   procedure Test_Convert is
      procedure Assert_Convert_Eq (From : Digit_T; To : String) is
      begin
         pragma Assert (Convert (From) = To);
      end Assert_Convert_Eq;

      procedure Assert_Convert_Eq (From : String; To : Digit_T) is
      begin
         pragma Assert (Convert (From) = To);
      end Assert_Convert_Eq;
   begin
      Assert_Convert_Eq (0, "zero");
      Assert_Convert_Eq (1, "one");
      Assert_Convert_Eq (2, "two");

      Assert_Convert_Eq ("one", 1);
      Assert_Convert_Eq ("four", 4);
      Assert_Convert_Eq ("five", 5);
   end Test_Convert;


   -- User Inputs --

   type User_Input is new String;

   function Get return User_Input is
      use Ada.Text_IO;
   begin
      Put ("> ");
      return User_Input (Get_Line);
   end Get;

   function To_Digit (U : User_Input) return Digit_T is
   begin

      if U'Length = 1 and then U (U'First) in '0' .. '9' then
         return Digit_T'Value (String (U));
      else
         --  all others case - might fail if input is not a digit
         --  textual representation.
         return Convert (String (U));
      end if;

   end To_Digit;

   function Eq (U1, U2 : User_Input) return Boolean is
   begin
      return To_Digit (U1) = To_Digit (U2);
   end Eq;

   function Transform (U : User_Input) return User_Input is
   begin

      declare
         U_Str : constant String := String (U);
         
         function Transform (S : String) return String is
         begin
            if S'Length = 1 and then S (S'First) in '0' .. '9' then
               return Convert (Digit_T'Value (S));
            else
               declare
                  function To_String (D : Digit_T) return String is
                     R : constant String := Digit_T'Image (D);
                  begin
                     return R (R'First + 1 .. R'Last);
                  end To_String;
               begin
                  return To_String (Convert (S));
               end;
            end if;
         end Transform;
      begin
         return User_Input (Transform (U_Str));
      end;

   end Transform;

   procedure Test_User_Input is
      procedure Assert_User_Input_Eq (U1, U2 : User_Input) is
      begin
         pragma Assert (Eq (U1, U2));
      end Assert_User_Input_Eq;
   begin
      Assert_User_Input_Eq ("one", "1");
      Assert_User_Input_Eq ("2", "2");
      Assert_User_Input_Eq ("9", "nine");

      pragma Assert (Transform ("8") =  "eight");
      pragma Assert (Transform ("eight") =  "8");
   end Test_User_Input;
begin
   Test_Convert;
   Test_User_Input;

   declare
      First_Iteration : Boolean := True;
      Prev_D : Digit_T;
   begin
      loop
         declare
            U : User_Input := Get;
            D : constant Digit_T := To_Digit (U);
         begin

            Ada.Text_IO.Put_Line (String (Transform (U)));
            
            pragma Warnings (Off, "may be referenced before it has a value",
                             Reason => "First_Iteration would then be False");
            if not First_Iteration and then D = Prev_D then
               Ada.Text_IO.Put_Line ("similar");
            end if;
            pragma Warnings (On, "may be referenced before it has a value");
            
            Prev_D := D;
            First_Iteration := False;
         end;
      end loop;
   end;
end Main;
