with Ada.Text_IO;

procedure Main is
   type ID_T is new Natural;
   Invalid_ID : constant ID_T := ID_T'Last;
   subtype Valid_ID_T is ID_T range ID_T'First .. ID_T'Pred (Invalid_ID);

   protected type P is
      procedure Set (ID : Valid_ID_T);
      function Get return ID_T;
   private
      ID : ID_T := Invalid_ID;
   end P;

   protected body P is
      procedure Set (ID : Valid_ID_T) is
      begin
         P.ID := ID;
      end Set;

      function Get return ID_T is (P.ID);
   end P;

   P1 : P;

   task type T is
      entry Init (ID : Valid_ID_T; Wait_For : Duration);
   end T;

   task body T is
      ID : Valid_ID_T;
      Wait_For : Duration;
   begin
      accept Init (ID : Valid_ID_T; Wait_For : Duration) do
         T.ID := ID;
         T.Wait_For := Wait_For;
      end Init;

      loop
         delay Wait_For;
         P1.Set (ID);
         Ada.Text_IO.Put_Line (ID'Image);
      end loop;
   end T;

   T1, T2, T3 : T;

   Last_ID : ID_T := P1.Get;
begin
   T1.Init (1, 0.5);
   T2.Init (2, 1.0);
   T3.Init (3, 1.5);

   loop
      delay 1.0;
      
      declare
         Cur_ID : constant ID_T := P1.Get;
      begin
         if Last_ID /= Cur_ID then
            Last_ID := Cur_ID;
            Ada.Text_IO.Put_Line ("Main" & Cur_ID'Image);
         end if;
      end;
   end loop;
end Main;
